# coding=utf-8
from bscearth.utils.log import Log
import os

from earthdiagnostics.diagnostic import Diagnostic, DiagnosticOption, DiagnosticDomainOption, DiagnosticVariableOption
from earthdiagnostics.utils import Utils, TempFile
from earthdiagnostics.modelingrealm import ModelingRealms


class Rotation(Diagnostic):
    """
    Cuts a meridional or zonal section

    :original author: Virginie Guemas <virginie.guemas@bsc.es>
    :contributor: Javier Vegas-Regidor<javier.vegas@bsc.es>

    :created: September 2012
    :last modified: June 2016

    :param data_manager: data management object
    :type data_manager: DataManager
    :param startdate: startdate
    :type startdate: str
    :param member: member number
    :type member: int
    :param chunk: chunk's number
    :type chunk: int
    :param variable: variable's name
    :type variable: str
    :param domain: variable's domain
    :type domain: Domain
    """

    alias = 'rotate'
    "Diagnostic alias for the configuration file"

    def __init__(self, data_manager, startdate, member, chunk, domain, variableu, variablev, executable):
        Diagnostic.__init__(self, data_manager)
        self.startdate = startdate
        self.member = member
        self.chunk = chunk
        self.variableu = variableu
        self.variablev = variablev
        self.domain = domain
        self.executable = executable

    def __eq__(self, other):
        return self.startdate == other.startdate and self.member == other.member and self.chunk == other.chunk and \
            self.domain == other.domain and self.variableu == other.variableu and self.variablev == other.variablev

    def __str__(self):
        return 'Rotate variables Startdate: {0} Member: {1} Chunk: {2} Variables: {3}:{4} , ' \
               '{3}:{5}'.format(self.startdate, self.member, self.chunk, self.domain, self.variableu,
                                self.variablev)

    @classmethod
    def generate_jobs(cls, diags, options):
        """
        Creates a job for each chunk to compute the diagnostic

        :param diags: Diagnostics manager class
        :type diags: Diags
        :param options: variable, zonal, value, domain=ocean
        :type options: list[str]
        :return:
        """
        options_available = (DiagnosticVariableOption('variableu'),
                             DiagnosticVariableOption('variablev'),
                             DiagnosticDomainOption('domain', ModelingRealms.ocean),
                             DiagnosticOption('executable',
                                              '/home/Earth/jvegas/pyCharm/cfutools/interpolation/rotateUVorca'))
        options = cls.process_options(options, options_available)

        job_list = list()
        for startdate, member, chunk in diags.config.experiment.get_chunk_list():
            job_list.append(Rotation(diags.data_manager, startdate, member, chunk,
                                     options['domain'], options['variableu'], options['variablev'],
                                     options['executable']))
        return job_list

    def compute(self):
        """
        Runs the diagnostic
        """
        ufile = self.data_manager.get_file(self.domain, self.variableu, self.startdate, self.member, self.chunk)
        vfile = self.data_manager.get_file(self.domain, self.variablev, self.startdate, self.member, self.chunk)

        urotated = TempFile.get()
        vrotated = TempFile.get()

        namelist_file = self._create_namelist(ufile, urotated, vfile, vrotated)

        Utils.execute_shell_command('{0} {1}'.format(self.executable, namelist_file), Log.INFO)

        ufile_handler = Utils.openCdf(ufile)
        self._add_metadata_vars(ufile_handler, urotated)
        self._add_metadata_vars(ufile_handler, vrotated)
        ufile_handler.close()

        os.remove(ufile)
        os.remove(vfile)

        self.send_file(urotated, self.domain, self.variableu, self.startdate, self.member, self.chunk, grid='rotated')
        self.send_file(vrotated, self.domain, self.variablev, self.startdate, self.member, self.chunk, grid='rotated')

    def _create_namelist(self, ufile, urotated, vfile, vrotated):
        namelist_file = TempFile.get(suffix='')
        rotate_namelist = open(namelist_file, 'w')
        rotate_namelist.write('&nam_rotUV\n')
        rotate_namelist.write(' Ufilein = "{0}"\n'.format(ufile))
        rotate_namelist.write(' Uvarin = "{0}"\n'.format(self.variableu))
        rotate_namelist.write(' Vfilein = "{0}"\n'.format(vfile))
        rotate_namelist.write(' Vvarin = "{0}"\n'.format(self.variablev))
        rotate_namelist.write(' meshmask = "mask.nc"\n')
        rotate_namelist.write(' Ufileout = "{0}"\n'.format(urotated))
        rotate_namelist.write('Vfileout = "{0}"\n'.format(vrotated))
        rotate_namelist.writelines("/\n")
        rotate_namelist.close()
        return namelist_file

    def _add_metadata_vars(self, reference_file_handler, rotaded_file):
        rotated_handler = Utils.openCdf(rotaded_file)
        for var in reference_file_handler.variables.keys():
            if var not in rotated_handler.variables.keys() and var not in [self.variableu, self.variablev]:
                Utils.copy_variable(reference_file_handler, rotated_handler, var, True, True)
        rotated_handler.close()




